#version 330
#extension GL_ARB_separate_shader_objects : enable





layout(location = 0) smooth in vec2 _fin_fragTexCoord;

struct SFragmentInput
{
	vec2 fragTexCoord;
	bool IsFrontFace;
};

layout(location = 0) out vec4 _fout_outColor;

struct SFragmentOutput
{
	vec4 outColor;
};

 uniform sampler2D Texture;

layout( std140) uniform UToneMappingInfo
{
	vec4 ExpGammaSat;
} ToneMappingInfo;

void FragmentMain( in SFragmentInput  fInput,  out SFragmentOutput  fOutput  )
{



	vec2 uv = fInput.fragTexCoord;


	fOutput.outColor =  texture(Texture, uv) ;
	float exposure =  ToneMappingInfo.ExpGammaSat .x;
	vec3 gammaInv = 1. /  ToneMappingInfo.ExpGammaSat .yyy;
	float saturation =  ToneMappingInfo.ExpGammaSat .z;

	vec3 colorCorrected = fOutput.outColor.rgb;

	float greyIntensity = 0.2126 * colorCorrected.r + 0.7152 * colorCorrected.g + 0.0722 * colorCorrected.b;
	colorCorrected = max(colorCorrected * saturation + greyIntensity * (1.f - saturation), vec3(0, 0, 0));

	colorCorrected = 1.f - exp(- exposure * colorCorrected);
	colorCorrected = pow(colorCorrected, gammaInv);

	fOutput.outColor.rgb = colorCorrected;

}


void main()
{
	SFragmentInput fInput;
	SFragmentOutput fOutput;

	fInput.fragTexCoord = _fin_fragTexCoord;
	fInput.IsFrontFace = gl_FrontFacing;
	FragmentMain(fInput, fOutput);
	_fout_outColor = fOutput.outColor;
}
